from distutils.command import install
import importlib
from pathlib import Path
from loguru import logger
from omegaconf import OmegaConf
import os
import sys

import importlib

from .utils import gitclone

#install_dir = Path(__path__)
#local_path = Path.cwd() 

# to do: add a database (maybe just a text file) to track mappings from package name to install dir


#Path.expanduser()

# TO DO: check for an enviroment variable that can be used to override the install directory




def resolve_napm_path() -> str:
    """
    Returns the parent direectory into which dapm will "install" "pacakages"
    """
    # TO DO: better internal jargon to replace scarequotes
    # Todo: after first resolve, persist this to a config file or something like that and load from there. 
    napm_path = os.environ.get('NAPM_PATH')
    if napm_path is None:
        cache_dir = Path.home() / '.cache'
        napm_path = (cache_dir / 'napm').resolve()
        napm_path.mkdir(parents=True, exist_ok=True)
    napm_path = str(napm_path)
    if napm_path not in sys.path:
        sys.path.append(napm_path)
    return napm_path


def make_install_dir(dirname) -> str:
    """
    Creates the directory into which the package will be "installed".
    """
    parent = resolve_napm_path()
    install_dir = Path(parent) / dirname
    if not install_dir.exists():
        install_dir.mkdir(parents=True)
    else:
        logger.warning(f'{install_dir} already exists')
    install_dir = str(install_dir)
    return install_dir


def pseudoinstall_git_repo(
    package_url, 
    install_dir=None, 
    package_name=None,
    add_install_dir_to_path=False,
    ):
    if not package_name:
        package_name = package_url.split('/')[-1]
    if not install_dir:
        install_dir = make_install_dir(package_name)
    
    gitclone(package_url, install_dir)

    # test install was successful
    try:
        importlib.import_module(package_name)
    except ImportError as e:
        logger.error(f'{package_name} failed to import from dapm install dir')
        add_install_dir_to_path = True

    if add_install_dir_to_path:
        sys.path.append(install_dir)
        logger.debug(f"Added {install_dir} to sys.path")

    try:
        importlib.import_module(package_name)
    except ImportError as e:
        logger.error(f"Failed to import {package_name}, napm 'install' failed")
        raise e



import subprocess
#import os
#import sys
#import importlib
#import omegaconf

from loguru import logger

# should we use an SDK for this instead? Maybe https://gitpython.readthedocs.io/en/stable/tutorial.html ?
def gitclone(
    repo_url, 
    tgt_dir,
    ):
    """
    Alias for calling git clone.
    """
    #cmd = ['git', 'clone']
    #cmd += [url, tgt_dir]
    cmd = ['git', 'clone', repo_url, tgt_dir]
    res = subprocess.run(cmd, stdout=subprocess.PIPE).stdout.decode('utf-8')
    logger.debug(res)




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['napm']

package_data = \
{'': ['*']}

install_requires = \
['gdown>=4.4.0,<5.0.0',
 'loguru>=0.6.0,<0.7.0',
 'omegaconf>=2.1.1,<3.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'napm',
    'version': '0.1.1',
    'description': 'Not A Package Manager.',
    'long_description': None,
    'author': 'David Marx',
    'author_email': 'david.marx84@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from .pseudo_install import resolve_napm_path 
from pathlib import Path
import sys
from loguru import logger

def populate_pythonpaths():
    """
    If packages have been previously "installed" with napm, this will add the "install" directories to the python path.
    """
    napm_path = Path(resolve_napm_path())
    sys.path.append(napm_path)
    #for subdir in napm_path.iterdir():
    #    if subdir.is_dir():
    #        subdir = str(subdir.resolve())
    #        logger.debug(subdir)
    #        sys.path.append(subdir)

if __name__ == "__main__":
    populate_pythonpaths()